
#include "cconfigdispatcher.h"

CConfigDispatcher::CConfigDispatcher(QObject *parent)
 : QObject(parent)
{
  connect(this, SIGNAL(ErrorOpenConfigFile(QString&)), this, SLOT(OnErrorOpenFile(QString&)));
  connect(this, SIGNAL(ErrorReadingFile(QString&, int&, int&)), this, SLOT(OnErrorReadingFile(QString&, int&, int&)));
  connect(this, SIGNAL(ErrorConfigFile(QString)), this, SLOT(OnErrorConfigFile(QString)));
}


CConfigDispatcher::~CConfigDispatcher()
{
  qDeleteAll(Separators.begin(), Separators.end());
}

void CConfigDispatcher::Open(QString FileName)
{
  ConfigFileName = FileName;
  ConfigFile.setFileName(FileName);
  if (ConfigFile.open(QFile::ReadOnly))
  {
    ParseDocument();
  }
  else 
  {
    ConfigFile.close(); 
    QString Message = "Не удалось открыть файл конфигурации " + FileName;
    emit ErrorOpenConfigFile(Message);
  }
  ConfigFile.close();
}

void CConfigDispatcher::OnErrorOpenFile(QString & message)
{
  qDebug(message.toAscii());
}

void CConfigDispatcher::ParseDocument()
{
  QString errorMessage;
  int ErrorLine, ErrorColumn;  
  if (!ConfigDocument.setContent(&ConfigFile, &errorMessage, &ErrorLine, &ErrorColumn))
  {
   emit ErrorReadingFile(errorMessage, ErrorLine, ErrorColumn);
   return;
  }
  QDomElement Root = ConfigDocument.documentElement();
  if (Root.nodeName() == "CONFIG")
  {
   qDebug("Успешная загрузка файла конфигурации");
    for(QDomNode Node = Root.firstChild(); !Node.isNull(); Node = Node.nextSibling())
    {
       if (Node.nodeName() == "SEPARATORS")
       {
         ProcessSeparatorsTag(Node);
       }

       if (Node.nodeName() == "OPTIONS")
       {
         ProcessOptionsTag(Node);
       }
    }
  }
  else
  {
    emit ErrorConfigFile(tr("Некорректный файл конфигурации"));
  }
   
}

void CConfigDispatcher::OnErrorReadingFile(QString & message, int & ErrorLine, int & ErrorColumn)
{
  qDebug(message.toAscii() + " %d, %d", ErrorLine, ErrorColumn);
}

void CConfigDispatcher::OnErrorConfigFile(QString message)
{
  qDebug(message.toAscii());
}

void CConfigDispatcher::ProcessSeparatorsTag(QDomNode Node)
{
    QDomNode CurrentNode;
    for(QDomNode CurrentNode = Node.firstChild(); !CurrentNode.isNull(); CurrentNode = CurrentNode.nextSibling())
    {
       if (CurrentNode.nodeName() == "INCLUDE")
       {
         QString FILE_PATH = GetAttributeFromNode(CurrentNode, "FILE");
         if (!FILE_PATH.isNull())
         {
            ProcessIncludeSeparator(FILE_PATH);
         }
       }

       if (Node.nodeName() == "SEPARATOR")
       {
         ProcessSeparatorNode(CurrentNode);
       }
    }
}

void CConfigDispatcher::ProcessSeparatorNode(QDomNode Node)
{
  CSeparatorNode* SeparatorNode = new CSeparatorNode(this);
  SeparatorNode->ProcessByNode(Node);
  Separators.append(SeparatorNode);
}

void CConfigDispatcher::ProcessIncludeSeparator(QString FileName)
{
   CSeparatorNode* SeparatorNode = new CSeparatorNode(this);
   if (SeparatorNode->ProcessByFile(FileName, false))
   {
     Separators.append(SeparatorNode);
   }
}


void CConfigDispatcher::ProcessOptionsTag(QDomNode Node)
{
  qDebug(Node.nodeName().toAscii());
}

QString CConfigDispatcher::GetAttributeFromNode(QDomNode Node, QString AttributeName)
{
  return Node.toElement().attribute(AttributeName, QString::null);
  /*QDomNamedNodeMap DomNamedNodeMap = Node.attributes();
  if (DomNamedNodeMap.contains(AttributeName))
  {
    return DomNamedNodeMap.namedItem(AttributeName).nodeValue();
  } else return QString::null;
*/
}

CSeparatorNode* CConfigDispatcher::GetSeparatorConfig(int i)
{
  return Separators[i];
}




