
#include "cseparatornode.h"

CSeparatorNode::CSeparatorNode(QObject* parent)
 : CCustomNode(parent)
{
   connect(this, SIGNAL(ErrorConfigFile(QString)), this, SLOT(OnErrorConfigFile(QString)));
   connect(this, SIGNAL(ProcessNode()), this, SLOT(OnProcessNode()));
   connect(this, SIGNAL(FileCreated()), this, SLOT(OnFileCreated()));
   connect(this, SIGNAL(ErrorReadingFile(QString&, int&, int&)), this, SLOT(OnErrorReadingFile(QString&, int&, int&)));
   connect(this, SIGNAL(ErrorOpenConfigFile(QString&)), this, SLOT(OnErrorOpenConfigFile(QString&)));
}


CSeparatorNode::~CSeparatorNode()
{
   qDeleteAll(Properties.begin(), Properties.end());
}

void CSeparatorNode::OnErrorOpenConfigFile(QString & message)
{
  qDebug(message.toAscii());
}

void CSeparatorNode::OnErrorReadingFile(QString & message, int & line, int & column)
{
  qDebug(message.toAscii());
}

void CSeparatorNode::OnErrorConfigFile(QString message)
{
  qDebug(message.toAscii());
}

void CSeparatorNode::OnFileCreated()
{
}

void CSeparatorNode::OnProcessNode()
{
  for(QDomNode Node = GetNode().firstChild(); !Node.isNull(); Node=Node.nextSibling())
  {
   if (Node.nodeName() == "PROPERTIES")
    {
      ProcessPropertiesNode(Node);
    }
    if (Node.nodeName() == "SEPARATE_SETUP")
    {
      ProcessSeparateSetupNode(Node);
    }

  }
}

void CSeparatorNode::ProcessPropertiesNode(QDomNode PropertiesNode)
{
 for(QDomNode Node = PropertiesNode.firstChild(); !Node.isNull(); Node=Node.nextSibling())
  {
    if (Node.nodeName() == "PROPERTY")
    {
      ProcessPropertyNode(Node);
    }
  }
}


QString CSeparatorNode::GetIPAddress()
{
  return GetAttributeValue("IP");
}

QString CSeparatorNode::GetName()
{
  return GetAttributeValue("NAME");
}

int CSeparatorNode::GetPort()
{
  int Port = GetAttributeValue("NAME").toInt(&ConvertOK, 10);
  if (!ConvertOK)
  { 
     return Port; 
  }
  else 
  {
   return 0;
  }
}

void CSeparatorNode::SetIPAddress(QString IP)
{
  SetAttributeValue("IP", IP);
}

void CSeparatorNode::SetPort(quint16 PORT)
{
  GetNode().toElement().setAttribute("PORT", PORT);
}

void CSeparatorNode::SetName(QString NAME)
{
  SetAttributeValue("NAME", NAME);
}

void CSeparatorNode::ProcessPropertyNode(QDomNode Node)
{
   CPropertyNode* Property = new CPropertyNode(this);
   Property->ProcessByNode(Node);
   Properties.append(Property);
}

CPropertyNode* CSeparatorNode::CreateNewProperty(QString CODE, int ValueCount)
{
  QDomElement PropertiesNode = GetNode().firstChildElement("PROPERTIES");
  QDomElement NewProperty = GetNode().ownerDocument().createElement("PROPERTY");
  PropertiesNode.appendChild(NewProperty);
  NewProperty.setAttribute("CODE", CODE);
  if (ValueCount > 0)
  {
	for(int i=0; i<ValueCount; i++)
        {
          QDomElement ValueNode = GetNode().ownerDocument().createElement("VALUE");
          QDomNode N = NewProperty.appendChild(ValueNode);
          if (N.isNull()) qDebug("NULL!");
          //N.setNodeValue("AAA");
          QString S = "Значение свойства установилось в " + N.nodeValue();
          qDebug(S.toAscii());
        }
  } 
  CPropertyNode* Property = new CPropertyNode(this, ValueCount);
  Property->ProcessByNode(NewProperty);
  Properties.append(Property);
  return Property;
}

CPropertyNode* CSeparatorNode::GetProperty(QString CODE)
{
   QList<CPropertyNode*>::iterator Property;
   for (Property = Properties.begin(); Property != Properties.end(); ++Property)
   {
     CPropertyNode* Current = *Property;
     if (Current->GetCode() == CODE) return Current;
   }
   return NULL;
}

void CSeparatorNode::DeleteProperty(QString CODE)
{
  CPropertyNode* PropertyForDelete = GetProperty(CODE);
  if (PropertyForDelete != NULL)
  {
    QDomDocument Doc = PropertyForDelete->GetNode().ownerDocument();
    QDomNode Node = PropertyForDelete->GetNode().parentNode();
    if (!Node.isNull())
    {
      Node = Node.removeChild(PropertyForDelete->GetNode());
      if (!Node.isNull()) qDebug("Deleted successfully");
    }
    int Removes = Properties.removeAll(PropertyForDelete);
    qDebug("Removed %d", Removes);
  }
}



void CSeparatorNode::ProcessRange(QDomNode Node, int ChannelIndex)
{
  int i = -1;
  for(QDomNode RangeNode = Node.firstChild(); !RangeNode.isNull(); RangeNode = RangeNode.nextSibling())
  {
    i++;
    QString SValue = GetAttributeValue(RangeNode, "Ls");
    FSeparateSetup.Obl[ChannelIndex][i].Ls = SValue.toDouble(&ConvertOK);
    if (!ConvertOK)
    {
      FSeparateSetup.Obl[ChannelIndex][i].Ls = 0.00;
      emit ErrorConfigFile("Could not convert to double");
    }

    SValue = GetAttributeValue(RangeNode, "Rs");
    FSeparateSetup.Obl[ChannelIndex][i].Rs = SValue.toDouble(&ConvertOK);

    if (!ConvertOK)
    {
      FSeparateSetup.Obl[ChannelIndex][i].Rs = 0.00;
      emit ErrorConfigFile("Could not convert to double");
    }

  }
}

void CSeparatorNode::ProcessSeparateSetupOblNode(QDomNode SepSetupOblNode)
{
  for(QDomNode Node = SepSetupOblNode.firstChild(); !Node.isNull(); Node=Node.nextSibling())
  {
    if (Node.nodeName() == "CHANNEL")
    {
       QString SChannelIndex = GetAttributeValue(Node, "INDEX");
       int ChannelIndex = SChannelIndex.toInt(&ConvertOK, 10);
       if (!ConvertOK)
       {
           emit ErrorConfigFile("Config error CSeparatorNode::ProcessSeparateSetupOblNode");
           return;
       } 
       ProcessRange(Node, ChannelIndex);
        
    }
  }
}

void CSeparatorNode::ProcessPrgNode(QDomNode PrgNode)
{
  for(QDomNode Node = PrgNode.firstChild(); !Node.isNull(); Node=Node.nextSibling())
  {
      if (Node.nodeName() == "CHANNEL")
      {
	QString SIndex = GetAttributeValue(Node, "INDEX");
        int Index = SIndex.toInt(&ConvertOK, 10);
        if (!ConvertOK)
        {
           emit ErrorConfigFile("Cannot convert to int attribute INDEX in sort limits in separate setup");
           continue;
        }
        for (int i=0; i<MAX_PR; i++)
        {
          QString Value = GetAttributeValue(Node, "p", i);
          FSeparateSetup.Prg[Index][i] = Value.toDouble(&ConvertOK);
          if (!ConvertOK) 
          { 
            FSeparateSetup.Prg[Index][i] = -1.00; 
            emit ErrorConfigFile("Cannot convert to double attribute p in sort limits in separate setup");
            continue;
          }  
        } 
         
      }
  }
}

void CSeparatorNode::ProcessSeparateSetupNode(QDomNode SepSetupNode)
{
 QString AttrName;
 QString AttrValue;
 int i;
 for(QDomNode Node = SepSetupNode.firstChild(); !Node.isNull(); Node=Node.nextSibling())
  {
    if (Node.nodeName() == "OBL")
    {
      ProcessSeparateSetupOblNode(Node);
    }

    if (Node.nodeName() =="GmZ")
    {
       for (i = 0; i<MAX_CH-1; i++)
       {
          AttrValue = GetAttributeValue(Node, "g", i);
          FSeparateSetup.GmZ[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.GmZ[i] = 0.00;
       }
    }

    if (Node.nodeName() =="SH1")
    {
       for (i = 0; i<6; i++)
       {
          AttrValue = GetAttributeValue(Node, "s", i);
          FSeparateSetup.SH1[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.SH1[i] = 0.00;
       }
    }

    if (Node.nodeName() =="SH2")
    {
       for (i = 0; i<6; i++)
       {
          AttrValue = GetAttributeValue(Node, "s", i);
          FSeparateSetup.SH2[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.SH2[i] = 0.00;
       }
    }

    if (Node.nodeName() =="SH3")
    {
       for (i = 0; i<6; i++)
       {
          AttrValue = GetAttributeValue(Node, "s", i);
          FSeparateSetup.SH3[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.SH3[i] = 0.00;
       }
    }

    if (Node.nodeName() =="KH1")
    {
       for (i = 0; i<2; i++)
       {
          AttrValue = GetAttributeValue(Node, "k", i);
          FSeparateSetup.KH1[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.KH1[i] = 0.00;
       }
    }

    if (Node.nodeName() =="KH2")
    {
       for (i = 0; i<2; i++)
       {
          AttrValue = GetAttributeValue(Node, "k", i);
          FSeparateSetup.KH2[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.KH2[i] = 0.00;
       }
    }

    if (Node.nodeName() =="KH3")
    {
       for (i = 0; i<2; i++)
       {
          AttrValue = GetAttributeValue(Node, "k", i);
          FSeparateSetup.KH3[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.KH3[i] = 0.00;
       }
    }
    if (Node.nodeName() =="K_PR_MIN")
    {
      AttrValue = GetAttributeValue(Node, "Value");
      FSeparateSetup.kPrMin = AttrValue.toDouble(&ConvertOK);
      if (!ConvertOK) FSeparateSetup.kPrMin = 0.00;
    }

    if (Node.nodeName() =="K_PR_MAX")
    {
      AttrValue = GetAttributeValue(Node, "Value");
      FSeparateSetup.kPrMax = AttrValue.toDouble(&ConvertOK);
      if (!ConvertOK) FSeparateSetup.kPrMax = 0.00;
    }

    if (Node.nodeName() =="Prg")
    {
      ProcessPrgNode(Node);
    }

    if (Node.nodeName() =="Tiz")
    {
       for (i = 0; i<MAX_GR-1; i++)
       {
          AttrValue = GetAttributeValue(Node, "t", i);
          FSeparateSetup.Tiz[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.Tiz[i] = 0.00;
       }
    }

    if (Node.nodeName() =="fH12")
    {
      AttrValue = GetAttributeValue(Node, "Value");
      FSeparateSetup.fH12 = AttrValue.toDouble(&ConvertOK);
      if (!ConvertOK) FSeparateSetup.fH12 = 0.00;
    }

    if (Node.nodeName() =="fOtb")
    {
      AttrValue = GetAttributeValue(Node, "Value");
      FSeparateSetup.fOtb = AttrValue.toDouble(&ConvertOK);
      if (!ConvertOK) FSeparateSetup.fOtb = 0.00;
    }

    if (Node.nodeName() =="MaxG1")
    {
      AttrValue = GetAttributeValue(Node, "Value");
      FSeparateSetup.MaxG1 = AttrValue.toDouble(&ConvertOK);
      if (!ConvertOK) FSeparateSetup.MaxG1 = 0.00;
    }

    if (Node.nodeName() =="MinG1")
    {
      AttrValue = GetAttributeValue(Node, "Value");
      FSeparateSetup.MinG1 = AttrValue.toDouble(&ConvertOK);
      if (!ConvertOK) FSeparateSetup.MinG1 = 0.00;
    }

    if (Node.nodeName() =="GCol")
    {
      AttrValue = GetAttributeValue(Node, "Value");
      FSeparateSetup.GCo = AttrValue.toDouble(&ConvertOK);
      if (!ConvertOK) FSeparateSetup.GCo = 0.00;
    }

    if (Node.nodeName() =="KRuch")
    {
      AttrValue = GetAttributeValue(Node, "Value");
      FSeparateSetup.KRuch = AttrValue.toDouble(&ConvertOK);
      if (!ConvertOK) FSeparateSetup.KRuch = 0.00;
    }

    if (Node.nodeName() =="Usl")
    {
      
    }

    if (Node.nodeName() =="TOtb")
    {
      AttrValue = GetAttributeValue(Node, "Value");
      FSeparateSetup.TOtb = AttrValue.toDouble(&ConvertOK);
      if (!ConvertOK) FSeparateSetup.TOtb = 0.00;
    }

    if (Node.nodeName() =="K_im")
    {
       for (i = 0; i<MAX_CH-1; i++)
       {
          AttrValue = GetAttributeValue(Node, "u", i);
          FSeparateSetup.K_im[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.K_im[i] = 0.00;
       }
    }

    if (Node.nodeName() =="B_im")
    {
       for (i = 0; i<MAX_CH-1; i++)
       {
          AttrValue = GetAttributeValue(Node, "u", i);
          FSeparateSetup.B_im[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.B_im[i] = 0.00;
       }
    }

    if (Node.nodeName() =="K_zd")
    {
       for (i = 0; i<MAX_CH-1; i++)
       {
          AttrValue = GetAttributeValue(Node, "u", i);
          FSeparateSetup.K_zd[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.K_zd[i] = 0.00;
       }
    }

    if (Node.nodeName() =="B_zd")
    {
       for (i = 0; i<MAX_CH-1; i++)
       {
          AttrValue = GetAttributeValue(Node, "u", i);
          FSeparateSetup.B_zd[i] = AttrValue.toDouble(&ConvertOK);
          if (!ConvertOK) FSeparateSetup.B_zd[i] = 0.00;
       }
    }


  }
}

TSeparateSetup CSeparatorNode::GetSeparateSetup()
{
  return FSeparateSetup;
}

void CSeparatorNode::SetSeparateSetup(TSeparateSetup SepSetup)
{
  FSeparateSetup = SepSetup;
}

void CSeparatorNode::SaveSepSetup()
{
}

CPropertyNode * CSeparatorNode::operator [ ](int i)
{
  return Properties[i];
}

CPropertyNode * CSeparatorNode::operator [ ](QString CODE)
{
   return GetProperty(CODE);
}



