const quint8 C_GET_STATE		= 0xA;
const quint8 C_SET_SPECTRUM		= 0x10;
const quint8 C_GET_SPECTRUM		= 0x11;
const quint8 C_SETUP_SEPARATE		= 0x20;
const quint8 C_SET_DEU			= 0x30; 
const quint8 C_SET_P_DEU		= 0x63;
const quint8 C_START_SEPARATE		= 0x22;
const quint8 C_GET_SEPARATE_STATE	= 0x21;
const quint8 C_STOP_SEPARATE		= 0x23;
const quint8 C_GET_CONTROL_SPECTRUM	= 0x24;
const quint8 C_GET_GISTOGRAMM		= 0x25;
const quint8 C_SET_PUW			= 0x28;
const quint8 C_SET_REN			= 0x31;
const quint8 C_GET_REN			= 0x32;
const quint8 C_EXPOSITION_ON		= 0x40;
const quint8 C_EXPOSITION_OFF		= 0x41;
const quint8 C_SET_TERMO		= 0x37;
const quint8 C_GET_TERMO		= 0x38;
const quint8 C_SET_RGU			= 0x35;
const quint8 C_START_TEST_IM		= 0x0B;
const quint8 C_STOP_TEST_IM		= 0x0F;
const quint8 C_START_PUW		= 0x27;
const quint8 C_STOP_PUW			= 0x29;
const quint8 C_REN_ON			= 0x33;
const quint8 C_REN_OFF			= 0x34;
const quint8 C_SEPARATOR_ON		= 0x50;
const quint8 C_SEPARATOR_OFF		= 0x51;
const quint8 C_IW_ON			= 0x52;
const quint8 C_IW_OFF			= 0x53;
const quint8 C_LIGHT_ON			= 0x54;
const quint8 C_LIGHT_OFF		= 0x55;
const quint8 C_SET_RGU_EX		= 0x56;
const quint8 C_GET_RGU_EX		= 0x57;
const quint8 C_SET_M50			= 0x65;
const quint8 C_GET_M50			= 0x66;
const quint8 C_INIT_ADA			= 0xC;
const quint8 C_SET_LOG			= 0x68;
const quint8 C_GET_DOOR_BLOCKING	= 0x67;  

const int MAX_CH  = 4;
const int MAX_GR  = 5;
const int MAX_OBL = 6;
const int MAX_PR  = 6;

typedef unsigned short TSpectrumData [256];

enum TSeparatorState
{
  stWaitingForCommand,
  stGettingSpectrum,
  stSeparating,
  stWorkingWithRengen,
  stWorkingWithTemp,
  stTesting, 
  stUndefined
};

struct TControlSpectrumData
{  
   int AccamulationTime;
   TSpectrumData Spectrum;
};

struct TSpectrumRange
{
   double Ls;
   double Rs;
};

typedef TSpectrumRange TSeparateRanges [MAX_CH-1][MAX_OBL-1];
typedef double TGmz[MAX_CH-1];

struct TSeparateSetup
{
    TSeparateRanges Obl;
    TGmz GmZ;
    double SH1[6];
    double SH2[6];
    double SH3[6];
    double KH1[2];
    double KH2[2];
    double KH3[2];
    double Prg[MAX_CH-1][MAX_PR-1];
    double Tiz[MAX_GR-1];
    double fH12;
    double fOtb;
    double MaxG1;
    double MinG1;
    double GCo;
    double KRuch;
    double Usl[MAX_CH-1];
    double TOtb;
    double K_im[MAX_CH-1];
    double B_im[MAX_CH-1];
    double K_zd[MAX_CH-1];
    double B_zd[MAX_CH-1];
    double kPrMin;
    double kPrMax;
};

struct TSeparateState
{
   double Concentrates[MAX_CH]; 
   double Tails[MAX_CH];
   double StonesCount[MAX_CH];
   double PAverage[MAX_CH - 1];
   double PAverageConcentrate [MAX_CH-1];
   double PAverageTail[MAX_CH-1];
   double CurrentStoneP[MAX_CH-1];
   double CurrentStoneP1[MAX_CH-1];
   double CurrentStoneP2[MAX_CH-1];
   double CurrentStoneP3[MAX_CH-1];
   double CurrentStoneMeasureTime[MAX_CH-1];
   double MeasureTimeGistogramm[MAX_CH-1][MAX_GR-1];
   double ErrorIM;
};

typedef int TGistogramm[200];

struct TXRayTubePower
{
    unsigned short U;
    unsigned short I;
};

struct THeatSensingDeviceData
{
    double LowLimit;
    double HighLimit;
    double CurrentTemperature;
};

enum TRGUState {rsUndefined, rsUp, rsDown, rsError};

struct TM50State
{
    bool ExpositionOn;
    bool TubeI;
    bool TubeU;
    bool LowPowerU;
    bool Regime;
    bool TermoBlocking;
    bool OutOfTermalLimit;
    bool BlockOff;
};
